// $Header: /u/s/o/solomon/java/rooms.tmp/RCS/DialogWindow.java,v 1.8 1996/06/20 19:23:44 solomon Exp solomon $
// Copyright (c) 1996, Marvin Solomon

package rooms;

import java.awt.Dialog;
import java.awt.Color;
import java.awt.Frame;
import java.awt.Font;
import java.awt.TextField;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Component;
import java.awt.Button;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Event;
import java.awt.TextArea;

// A pop-up window to confirm an action
// NB:  Should be "extends Dialog", but that requires the construtor
// to get a Frame instance, which makes it impossible to use from an
// Applet.
public class DialogWindow extends Frame implements Constants {
	private TextField comment = null;
	private String cmd;
	private Button confirmButton, cancelButton, resetButton;
	private int type;

	// Values for the "type" parameter of the constructor
	static final int CREATE = 1;
	static final int DELETE = 2;
	static final int HELP = 3;
	static void debug(String s) { Rooms.debug(s); }

	public DialogWindow(int type, String msg, String cmd) {
		super("Reserve a Room");
		this.type = type;
		this.cmd = cmd;
		setBackground(Color.white);
		setForeground(Color.black);

		GridBagLayout gridBag = new GridBagLayout();
		setLayout(gridBag);
		GridBagConstraints c = new GridBagConstraints();
		
		// Defaults for all components
		c.weightx = c.weighty = 0;				// default (all equal)
		c.anchor = GridBagConstraints.CENTER;	// center in available space

		// Summary info about reservation
		c.gridwidth = GridBagConstraints.REMAINDER; // last in row
		c.anchor = GridBagConstraints.WEST;		// flush left
		Component label;
		if (type == HELP) {
			TextArea textArea = new TextArea(msg);
			textArea.setEditable(false);
			textArea.setFont(Rooms.defaultFont);
			label = textArea;
		}
		else {
			int i = 0, j;
			String indent;
			for (i = 0; (j = msg.indexOf('\n', i)) >= 0; i = j+1) {
				indent = "";
				if (j-i > 60) {
					// line too long; break it in two
					int k = msg.indexOf(' ', i+60);
					if (k > 0 && k < j) {
						label = new Label(msg.substring(i,k));
						label.setFont(Rooms.defaultFont);
						gridBag.setConstraints(label, c);
						add(label);
						i = k+1;
					}
					indent = "   ";
				}
				label = new Label(indent + msg.substring(i,j));
				label.setFont(Rooms.defaultFont);
				gridBag.setConstraints(label, c);
				add(label);
			}
			label = new Label(msg.substring(i));
			label.setFont(Rooms.defaultFont);
		}
		gridBag.setConstraints(label, c);
		add(label);
		c.anchor = GridBagConstraints.CENTER;		// restore defaults

		// Prompt
		switch (type) {
			case CREATE:
				label = new Label("Reason:");
				label.setFont(Rooms.defaultFont);
				break;
			case DELETE:
				label = new Label(
					"Do you really want to delete this reservation?");
				label.setFont(Rooms.defaultFont);
				break;
			case HELP:
				label = null;
				break;
			default: throw new IllegalArgumentException("unknown Dialog type");
		}
		if (label != null) {
			c.anchor = GridBagConstraints.EAST;			// cozy up to next item
			c.gridwidth = 1;							// first in row
			gridBag.setConstraints(label, c);
			c.anchor = GridBagConstraints.CENTER;		// restore defaults
			add(label);
		}

		if (type == CREATE) {
			// Place to type reason
			comment = new TextField();
			comment.setFont(Rooms.defaultFont);
			c.fill = GridBagConstraints.HORIZONTAL;	// expand to available space
			c.gridwidth = GridBagConstraints.REMAINDER;	// last in row
			c.weightx = 1.0; c.weighty = 1.0;			// grab all extra space
			gridBag.setConstraints(comment, c);
			c.fill = GridBagConstraints.NONE;			// restore defaults
			c.weightx = 0.0; c.weighty = 0.0;			// restore defaults
			add(comment);
		}

		// Button box
		Panel panel = new Panel();
		switch (type) {
			case CREATE:
				confirmButton = new Button("Make Reservation");
				break;
			case DELETE:
				confirmButton = new Button("Delete Reservation");
				break;
			case HELP:
				confirmButton = null;
				break;
		}
		if (confirmButton != null) {
			confirmButton.setFont(Rooms.defaultFont);
			panel.add(confirmButton);
		}
		cancelButton = new Button(type == HELP ? "Dismiss" : "Cancel");
		cancelButton.setFont(Rooms.defaultFont);
		panel.add(cancelButton);
		if (type == CREATE) {
			resetButton = new Button("Reset");
			resetButton.setFont(Rooms.defaultFont);
			panel.add(resetButton);
		}
		gridBag.setConstraints(panel, c);
		add(panel);

		if (type == HELP)
			resize(625, 325);
		else
			resize(600, 250);
	}

	public boolean action(Event event, Object arg) {
		if (event.target == confirmButton
			|| event.target instanceof TextField)
		{
			if (type == CREATE) {
				Rooms.contactServer(cmd, comment.getText());
				comment.selectAll();
			}
			else
				Rooms.contactServer(cmd, null);
			hide();
			Rooms.disabled = false;
			return true;
		}
		if (event.target == cancelButton) {
			if (comment != null)
				comment.selectAll();
			hide();
			if (type != HELP)
				Rooms.disabled = false;
			return true;
		}
		if (event.target == resetButton) {
			comment.setText("");
			return true;
		}
		return false;
	}
} // DialogWindow
