// $Header: /u/s/o/solomon/java/rooms.tmp/RCS/Parser.java,v 1.5 1996/06/20 19:23:44 solomon Exp solomon $
// Copyright (c) 1996, Marvin Solomon

package rooms;

import java.io.InputStream;
import java.io.DataInputStream;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;

// Parse a response from the server and build appropriate reservations
abstract class Parser {
	static void debug(String s) { Rooms.debug(s); }

	// Parse stream "in" and call g.addReservation() for each reservation found
	// The data consists of the output of "rooms print -J", which precedes
	// each reservation with a line of the form
	//   !< 2310 36000 43200 0
	// and follows it with
	//   !>
	// Here "2310" is a room name, 36000 and 43200 are the starting
	// and ending times of the reservation (in seconds past midnight),
	// and the last item is 1 for a deleted reservation and 0 for an active
	// one.
	// Anything not bracketed by !< and !> is ignored.
	static void parse(DataInputStream in, ReservationGrid g) {
		StringTokenizer tok;
		String line, word, room, start, end, id=null, detail;
		boolean deleted, bad;
		int line_number = 0; // for debugging messages

		room = start = end = null; // seems to be required by javac (bug?)
		try {
			for (;;) {
				line = in.readLine();
				if (line == null)
					return;
				line_number++;
				tok = new StringTokenizer(line);

				// get next reservation
				if (!tok.hasMoreTokens()
					|| !"!<".equals(word = tok.nextToken()))
				{
					// debug("skipping line "+line);
					continue;
				}
				// parse header line:
				// !< 2310 36000 43200 0 1234
				bad = deleted = false;
				try {
					room = tok.nextToken();
					start = tok.nextToken();
					end = tok.nextToken();
					deleted = "1".equals(tok.nextToken());
					id = tok.nextToken();
				}
				catch (NoSuchElementException e) {
					System.out.println(
						"not enoungh tokens on line "+line_number+" "+line);
					bad = true;
				}
				// now grab everything up to next !< line
				detail = "";
				for (;;) {
					line = in.readLine();
					if (line == null) {
						System.out.println("premature eof");
						return;
					}
					if (line.charAt(0) == '!') {
						// really should check for !>
						break;
					}
					detail += line+"\n";
				}
				// finally, record the reservation
				if (bad || deleted) {
					// debug("skipping reservation");
					continue;
				}
				g.addReservation(id,room,start,end,detail);
			} // for(;;)
		}
		catch(Throwable ex) { ex.printStackTrace(); }
	} // Parser.parse
} // Parser
