// $Header: /u/s/o/solomon/java/rooms.tmp/RCS/Rooms.java,v 1.38 1996/06/20 19:23:44 solomon Exp solomon $
// Copyright (c) 1996, Marvin Solomon
package rooms;

import java.applet.Applet;
import java.util.Date;
import java.net.URL;
import java.net.URLConnection;
import java.io.PrintStream;
import java.io.DataInputStream;
import java.io.OutputStream;
import java.awt.*;

// This is a "coordinator" class that allows communication between the
// component panes of the Rooms application.  All data members are
// static and the class is abstract, so it behaves essentially like
// a bunch of global variables and functions.
public abstract class Rooms {
	/*
	public static void main(String[]args) {
		Frame f = new Frame();
		MonthChooser m = new MonthChooser();
		m.init();
		f.add(m);
		f.pack();
		f.show();
	}
	*/
	// used to simulate modal dialogs
	static boolean disabled = true;

	// Default font size (big enough so '0' is 7 pixels wide)
	static int fontSize;

	// Barrier to block each of the applets until all have started
	static Barrier barrier = new Barrier(5);

	// set from applet parameters
	static String ticket = null;		// if non-null, ticket to make updates
	static boolean debugFlag = true;
	static String cgiBin	// URL of the cgi-bin directory to contact for info.
		= "http://www.cs.wisc.edu/cgi-rooms";
	static void debug(String s) {
		if (debugFlag) {
			String t = Thread.currentThread().getThreadGroup().getName();
			int n = t.indexOf(".");
			if (n > 0) t = t.substring(n+1);
			System.out.println(t + ": " + s);
		}
	}

	// Fonts (set by ButtonFrame.init())
	static Font defaultFont, smallFont;

	// Applets
	static MonthChooser chooser;
	static CalGrid calendar;
	static ReservationGrid grid;
	static ButtonFrame buttonBox;
	static TextArea info;
	//static Label title;

	// At any one time there may be "current selection", a "current choice",
	// or neither (but not both).  The current selection represents a
	// selected existing reservation.  It is recorded in currentSelection,
	// which is either a prointer to a Reservation object or is null.
	// The current choice represents a new reservation being created.
	// It is recorded as three strings indicating the room and start and
	// end times.  The string resRoom is null if there is no current
	// choice.
	static Reservation currentSelection = null;
	static String resRoom = null, resStart, resEnd;

	static void setChoice(String room, String start, String end) {
		resRoom = room;
		resStart = start;
		resEnd = end;
		if (ticket != null)
			buttonBox.reserveButton.enable();
	}
	static void clearChoice() {
		resRoom = null;
		if (buttonBox.reserveButton != null)
			buttonBox.reserveButton.disable();
	}
	static void setSelection(Reservation r) {
		currentSelection = r;
		if (ticket != null)
			buttonBox.deleteButton.enable();
	}
	static void clearSelection() {
		currentSelection = null;
		buttonBox.deleteButton.disable();
	}

	static private void setCursor(int c) {
		Component o;
		for (o = grid; o != null; o = o.getParent()) {
			if (o instanceof Frame) {
				((Frame)o).setCursor(c);
				break;
			}
		}
		for (o = calendar; o != null; o = o.getParent()) {
			if (o instanceof Frame) {
				((Frame)o).setCursor(c);
				break;
			}
		}
	}

	// This procedure is called whenever the state of any of the
	// components may have changed.  It updates the others to be
	// consistent.
	static synchronized void update() {
		int m = chooser.month, y = chooser.year;

		// Only update the calendar if it changed months
		//debug("Rooms.update: m "+calendar.curMon+"->"+m+" y "
		//	+calendar.curYear+"->"+y);
		if (m != calendar.curMon || y != calendar.curYear)
			calendar.setMonth(m,y);

		// Always refresh the grid, regardless whether anything else
		// changed, for two reasons:  To initialize it the first time,
		// and to allow any click in the chooser or calendar to force
		// a re-read of current reservations.
		setCursor(Frame.WAIT_CURSOR);
		grid.setDate(calendar.shortDate());
		setCursor(Frame.DEFAULT_CURSOR);
		calendar.repaint();
		// Sizes
		/*
		debug("chooser "+chooser.minimumSize());
		debug("calendar "+calendar.minimumSize());
		debug("grid "+grid.minimumSize());
		debug("buttonBox "+buttonBox.minimumSize());
		debug("info "+info.minimumSize());
		*/
	}

	// Contact the server to make a reservation given the command
	// (e.g., "reserve 4209 11:00 13:00 3/11/96") and reason
	// (e.g., "boring meeting\n").
	// Return true if the addition was successful.
	// Show information (e.g. error messages) in the info component.
	static void contactServer(String cmd, String reason) {
		// Creating a URL connects to the server and sends the arguments.
		String cmdLine = cgiBin + "/rooms.pl?-Abasic+" + ticket + "+-e";
		/* Add this for debugging the server
		if (debugFlag)
			cmdLine += "+-D+/u/s/o/solomon/junk/log+-h+gjetost";
		*/
		URL url;
		URLConnection conn;

		if (cmd.startsWith("reserve")) {
			if (reason != null)
				reason = reason.trim();
			if (reason == null || reason.equals("")) {
				info.setText(
					"You MUST type a comment to describe the reservation");
				return;
			}
		}
		StringBuffer msg = new StringBuffer();
		setCursor(Frame.WAIT_CURSOR);
		try { // body of contactServer
			// Open a connection to the server
			debug("URL " +cmdLine);
			try {
				url = new URL(cmdLine);
				conn = url.openConnection();
				conn.setUseCaches(false);
				conn.setDoOutput(true);
				conn.setDoInput(true);
				conn.setAllowUserInteraction(true);
			}
			catch (Throwable e) {
				msg.append("exception connecting to server ("+cmdLine+"): "
					+e+"\n");
				e.printStackTrace();
				return;
			}

			// Send the reservation command and comment to the output stream.
			// Important:  conn.getOutputStream() must be called before
			// conn.getInputStream() or conn.connect() to convince the
			// library to use the POST method rather than GET.
			try {
				PrintStream outStream = new PrintStream(conn.getOutputStream());
				debug("cmd: "+cmd);
				outStream.println(cmd);
				if (reason != null) {
					debug("reason: " + reason);
					outStream.print(reason + "\n");
				}
				outStream.close();
			}
			catch (Throwable e) {
				msg.append("exception on writing:"+e+"\n");
				return;
			}

			// Read back the response from the input stream.
			try {
				DataInputStream inStream =
					new DataInputStream(conn.getInputStream());
				String inputLine;

				while (null != (inputLine = inStream.readLine())) {
					debug("<<<" + inputLine + ">>>");
					if (isInteresting(inputLine))
						msg.append(inputLine + "\n");
				}
				inStream.close();
			}
			catch (Throwable e) {
				msg.append("exception on reading:"+e+"\n");
				return;
			}
			// debug("result is >>>>>>\n" + msg.toString() + "<<<<<<");

			// Refresh display
			if (msg.toString().indexOf("has been made for you") >= 0
				|| msg.toString().indexOf("deleting reservation") >= 0)
			{
				// Operation was successful
				grid.setDate(calendar.shortDate());
			}
		} // body of contactServer
		finally {
			info.setText(msg.toString().trim());
			setCursor(Frame.DEFAULT_CURSOR);
		}
	} // contactServer

	// Filter out "noise" response lines from the server.
	private static final String[] boring = {
		"> > ",
		"Version",
		"type `help' for help",
		"new user is",
		"Please type a one-line comment",
		"Type q to give up"
	};

	private static boolean isInteresting(String msg) {
		if (msg == null || msg.equals("") || msg.startsWith("%"))
			return false;
		for (int i=0; i<boring.length; i++)
			if (msg.indexOf(boring[i]) >= 0)
				return false;
		return true;
	}
} // Rooms

/**
The Barrier class is used to make sure everything gets initialized in the
right order.  The class Rooms is an abstract class that never gets
instantiated.  It acts like a set of global variables and functions that
coordinate instances of the Applet classes, which are expected to
be instantiated as table entries (which visually show up as panes in the
window).  Since actions in one pane may invoke operations on the objects
in other panes, it is important that all objects are created, registered
with Rooms, and initialized before any of them start running.  Also,
there may be dependencies among the init() functions themselves (e.g.,
ButtonFrame.init() creates some fonts used in the initialization of the
other applets).  Thus we have the convention that each init() function
is bracketed by Rooms.barrier.start(i) and Rooms.barrier.finish(i), where
i is a unique "precedence" statically assigned to each applet.
The rule is that Rooms.barrier.start(i) is not allowed to return until
Rooms.barrier.finish(i-1) as been called, and Rooms.barrier.finish() is
not allowed to return until rooms.barrier.finish() has been called count
times.
*/
class Barrier {
	int value = 0, n;
	boolean go[];
	Barrier(int i) {
		n = i;
		go = new boolean[n];
		for (i=0;i<n;i++)
			go[i] = false;
		go[0] = true;
	}
	synchronized void start(int i) {
		if (!go[i]) {
			try {
				wait();
			}
			catch (InterruptedException e) {
				Rooms.debug("Wait interrupted: "+e);
				e.printStackTrace();
			}
		}
	}
	synchronized void finish(int i) {
		value++;
		if (value < n) {
			go[value] = true;
			notify();
		}
		while (value < n) {
			try {
				wait();
			}
			catch (InterruptedException e) {
				Rooms.debug("Wait interrupted: "+e);
				e.printStackTrace();
			}
		}
		notify();
	}
}
