#!/s/std/bin/perl

# $Header: /u/s/o/solomon/java/rooms.tmp/RCS/specialize.pl,v 1.7 1996/05/21 17:33:40 solomon Exp solomon $

# Copyright (c) 1996, Marvin Solomon

# 'specialize FILE' copies FILE to STDOUT, replacing all "macro calls"
# of the form $(XXX).
# $(DATE) is replaced with the current date (output of the "date" command).
# $(WARN) is replaced with a warning that output should not be edited.
# Any other $(XXX) is replaced with the value of XXX in
# the environment.  If XXX is not in the environment, print a message to STDERR
# and replace it with ?XXX.
# Lines between #ifdef XXX and #endif XXX are included only if XXX is
# in the environment.  #else is also allowed, but otherwise, This is very
# rudimentary:  They do not nest, and there is no #ifndef. #elseif, etc.

usage() unless @ARGV == 1;

$INPUT = $ARGV[0];

die "$INPUT: $!\n" unless open(INPUT,"<$INPUT");
$skipping = 0;

while (<INPUT>) {
	if (/^#endif/) {
		$skipping = 0;
		next;
	}
	if (/^#else/) {
		$skipping = !$skipping;
		next;
	}
	if (/^#ifdef\s*(\S*)/) {
		$arg = $1;
		$val = $ENV{$arg};
		$skipping = !defined $val;
		next;
	}
	next if $skipping;
	while (/\$\(([^)]*)\)/) {
		$arg = $1;
		# parsed as "$(arg)"

		# Special cases
		if ($arg eq "DATE") {
			$val = `date`;
			chomp($val);
		}
		elsif ($arg eq "WARN") {
			$val = 
				"This file was created from rooms-register.pl.tmpl."
				. "  Do not Edit!";
		}

		# Default: lookup arg in the environment
		else {
			$val = $ENV{$arg};
			unless (defined $val) {
				print STDERR "$arg not defined\n";
				$val = "?$arg";
			}
		}
		s/\$\($arg\)/$val/;
	}
	print;
}

exit 0;

sub usage {
	print STDERR "usage: $0 template\n";
	exit 1;
}
